#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include "repo.h"

#include "md5.h"

static const char *guess_repo_name(const char *path);
static int parse_args(int argc, char **argv);

static const char *repo_path, *repo_name;

int main(int argc, char **argv)
{
	if(parse_args(argc, argv) == -1) {
		return 1;
	}

	if(repo_init(repo_path) == -1) {
		return 1;
	}
	/* repo_cleanup(); */

	if(!repo_name) {
		if(!(repo_name = guess_repo_name(repo_path))) {
			return 1;
		}
	}

	printf("Serving %s from %s\n", repo_name, repo_path);

	/* DBG */
	{
		struct md5_state sum;
		int i, count = repo_num_files();
		for(i=0; i<count; i++) {
			struct repo_file *file = repo_file(i);
			memcpy(sum.sum, file->chksum, sizeof sum.sum);
			printf("%s\tmd5:%s size:%lu mtime:%s", file->path, md5_sumstr(&sum),
					(unsigned long)file->size, ctime(&file->mtime));
		}
	}

	return 0;
}

static const char *guess_repo_name(const char *path)
{
	static char pathbuf[PATH_MAX];
	char *last_slash;
	int len;

	if(!realpath(path, pathbuf)) {
		goto fail;
	}
	len = strlen(pathbuf);

	if(len > 0 && pathbuf[len - 1] == '/') {
		pathbuf[--len] = 0;
	}
	if(len <= 0) goto fail;

	if((last_slash = strrchr(pathbuf, '/'))) {
		return last_slash + 1;
	}
	return pathbuf;

fail:
	fprintf(stderr, "failed to resolve path %s while determining the repo name. Please specify an explicit name with -id\n", repo_path);
	return 0;
}

static void print_usage(const char *argv0)
{
	printf("Usage: %s [options] [repo path]\n", argv0);
	printf("options:\n");
	printf("  -id <name>: repo identifier (defaults to directory name)\n");
	printf("  -h,-help: print help and exit\n");
}

static int parse_args(int argc, char **argv)
{
	int i;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(strcmp(argv[i], "-id") == 0) {
				repo_name = argv[++i];
			} else if(strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "-help") == 0) {
				print_usage(argv[0]);
				exit(0);
			} else {
				fprintf(stderr, "invalid option: %s\n", argv[i]);
				return -1;
			}
		} else {
			if(repo_path) {
				fprintf(stderr, "unexpected argument: %s\n", argv[i]);
				return -1;
			}
			repo_path = argv[i];
		}
	}

	if(!repo_path) repo_path = ".";
	return 0;
}
